/**
 *  Arduino Code Drawing Robot ESP8266[Arduino code]
 *
 *  Antonios Chatzisavvas [st0933@icte.uowm.gr] & Panagiotis Pourgias [st0949@icte.uowm.gr] 
 *  UOWM ICTE [http://icte.uowm.gr/] & [http://arch.icte.uowm.gr/]
 *  August 2017
 *  
 *
 */

 
#include <SoftwareSerial.h> 
#include <Servo.h>

SoftwareSerial ESPserial(2, 3); // RX | TX

// setup servo
int servoPin = 8;
int PEN_DOWN = 80; // angle of servo when pen is down
int PEN_UP = 155;   // angle of servo when pen is up
Servo penServo;

float wheel_dia=64; //    # mm (increase = spiral out)
float wheel_base=119.5; //    # mm (increase = spiral in, ccw) 
int steps_rev=512; //        # 512 for 64x gearbox, 128 for 16x gearbox
int delay_time=2; //         # time between steps in ms

// Stepper sequence org->pink->blue->yel
int L_stepper_pins[] = {4, 6, 7, 5}; 
int R_stepper_pins[] = {9, 11, 12, 10};

int fwd_mask[][4] =  {{1, 0, 1, 0},
                      {0, 1, 1, 0},
                      {0, 1, 0, 1},
                      {1, 0, 0, 1}};

int rev_mask[][4] =  {{1, 0, 0, 1},
                      {0, 1, 0, 1},
                      {0, 1, 1, 0},
                      {1, 0, 1, 0}};


void setup() {
  randomSeed(analogRead(1)); 
  Serial.begin(115200);
  ESPserial.begin(115200);    
  Serial.println("");
  Serial.println("[Shapes: DRAWING ROBOT WIFI]:Remember to to set Both NL & CR in the serial monitor.");
  Serial.println("Ready");
  Serial.println("");    
  
  for(int pin=0; pin<4; pin++){
    pinMode(L_stepper_pins[pin], OUTPUT);
    digitalWrite(L_stepper_pins[pin], LOW);
    pinMode(R_stepper_pins[pin], OUTPUT);
    digitalWrite(R_stepper_pins[pin], LOW);
  }
  penServo.attach(servoPin);
  Serial.println("setup");
  
  penup();
  
  delay(1000);
}



int com;
int ste;


void loop(){

  if ( ESPserial.available() )   { 
      String temp = "";   //Serial read Data form ESP8266
      while(ESPserial.available()){
        temp += (char)ESPserial.read();
        delay(100);
      }
      Serial.print(temp);
      int com;
      com = temp.toInt();

      if (com == 1) {
        Serial.print("Square");
        pendown();
        for(int x=0; x<4; x++ ){
         forward(100);
         left(90);
        }  
        penup();
      }
      else if(com == 2){
        Serial.print("Triangle");
        pendown();
        for(int x=0; x<3; x++ ){
         forward(100);
         left(120);
        }  
        penup();
      }
      else if(com == 3){
        Serial.print("Circle");
        pendown();
        
        float rotation = 2.0;
        float distance = wheel_base * 3.1412 * rotation;
        int steps = step(distance);
        for(int step=0; step<steps; step++){
          for(int mask=0; mask<4; mask++){
            for(int pin=0; pin<4; pin++){
              digitalWrite(R_stepper_pins[pin], fwd_mask[mask][pin]);
              //digitalWrite(L_stepper_pins[pin], fwd_mask[mask][pin]); 
            }
            delay(delay_time);
          } 
        }   
        penup();
      }
      else if(com == 4){
       Serial.print("Hexagon");
        pendown();
        for(int x=0; x<6; x++ ){
          forward(50);
          left(60);
        }   
        penup();
      }      
  }
  //done();      // releases stepper motor
}


// ----- HELPER FUNCTIONS -----------
int step(float distance){
  int steps = distance * steps_rev / (wheel_dia * 3.1412); //24.61
  
  Serial.print(distance);
  Serial.print(" ");
  Serial.print(steps_rev);
  Serial.print(" ");  
  Serial.print(wheel_dia);
  Serial.print(" ");  
  Serial.println(steps);
  delay(1000);
  return steps;  
}


void forward(float distance){
  int steps = step(distance);
  Serial.println(steps);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(L_stepper_pins[pin], rev_mask[mask][pin]);
        digitalWrite(R_stepper_pins[pin], fwd_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }
}


void backward(float distance){
  int steps = step(distance);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(L_stepper_pins[pin], fwd_mask[mask][pin]);
        digitalWrite(R_stepper_pins[pin], rev_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }
}


void left(float degrees){
  float rotation = degrees / 360.0;
  float distance = wheel_base * 3.1412 * rotation;
  int steps = step(distance);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(R_stepper_pins[pin], rev_mask[mask][pin]);
        digitalWrite(L_stepper_pins[pin], rev_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }   
}


void right(float degrees){
  float rotation = degrees / 360.0;
  float distance = wheel_base * 3.1412 * rotation;
  int steps = step(distance);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(R_stepper_pins[pin], fwd_mask[mask][pin]);
        digitalWrite(L_stepper_pins[pin], fwd_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }   
}


void done(){ // unlock stepper to save battery
  for(int mask=0; mask<4; mask++){
    for(int pin=0; pin<4; pin++){
      digitalWrite(R_stepper_pins[pin], LOW);
      digitalWrite(L_stepper_pins[pin], LOW);
    }
    delay(delay_time);
  }
}


void penup(){
  delay(250);
  Serial.println("PEN_UP()");
  penServo.write(PEN_UP);
  delay(250);
}


void pendown(){
  delay(250);  
  Serial.println("PEN_DOWN()");
  penServo.write(PEN_DOWN);
  delay(250);
}

